#include <stdlib.h>
#include <string.h>

#include "Queue.h"
#include "utils.h"

queue_t*
q_create(unsigned int data_size, unsigned int max_size)
{
  queue_t* q = calloc(1, sizeof(*q));
  DIE(!q, "calloc queue failed");

  q->data_size = data_size;
  q->max_size = max_size;

  q->buff = malloc(max_size * sizeof(*q->buff));
  DIE(!q->buff, "malloc buffer failed");

  return q;
}

unsigned int
q_get_size(queue_t* q)
{
  return !q ? 0 : q->size;
}

/*
 * Intoarce 1 daca stiva este goala si 0 in caz contrar.
 */
unsigned int
q_is_empty(queue_t* q)
{
  /* TODO */
  return !q ? 1 : !q->size;
}

void*
q_front(queue_t* q)
{
  /* TODO */
  if (!q || !q->size)
    return NULL;

  return q->buff[q->read_idx];
}

bool
q_dequeue(queue_t* q)
{
  if (!q || !q->size)
    return false;

  q->read_idx = (q->read_idx + 1) % q->max_size;
  --q->size;
  return true;
}

bool
q_enqueue(queue_t* q, void* new_data)
{
  void* data;
  if (!q || q->size == q->max_size)
    return false;

  data = malloc(q->data_size);
  DIE(!data, "malloc data failed");
  memcpy(data, new_data, q->data_size);

  q->buff[q->write_idx] = data;
  q->write_idx = (q->write_idx + 1) % q->max_size;
  ++q->size;

  return true;
}

void
q_clear(queue_t* q)
{
  unsigned int i;
  /**
   * ex: [ x | . | . | x | x ]
   * parcurgeti de la pozitia 3 pana la 0 (folosind % max_size)
   */
  if (!q || !q->size)
    return;

  for (i = q->read_idx; i != q->write_idx; i = (i + 1) % q->max_size)
    free(q->buff[i]);

  q->read_idx = 0;
  q->write_idx = 0;
  q->size = 0;
}

void
q_free(queue_t* q)
{
  if (!q)
    return;

  q_clear(q);
  free(q->buff);
  free(q);
}
